--[[
	DailyAzeroth (C) Kruithne <kruithne@gmail.com>
	Licensed under GNU General Public Licence version 3.
	
	https://github.com/Kruithne/DailyAzerothAddon

	DailyAzeroth.lua - Core engine file for the addon.
]]

-- Core container for stuffs.
DailyAzeroth = {
	Name = "DailyAzeroth",
	Sorting = "Name",
	SortMethod = 1, -- 1 = Ascending, 2 = Descending,
	SortTabs = {},
	SelectCount = 0,
	ShowCompleted = false,
	SelectedFilter = "All"
};

local defaultFilter = {
	Name = "All",
	Colour =  {r = 1.0, g = 1.0, b = 1.0, colorStr = "ffffffff"}
};

local classes = {
	{"HUNTER", "Hunter"},
    {"WARLOCK", "Warlock"},
    {"PRIEST", "Priest"},
    {"PALADIN", "Paladin"},
    {"MAGE", "Mage"},
    {"ROGUE", "Rogue"},
    {"DRUID", "Druid"},
    {"SHAMAN", "Shaman"},
    {"WARRIOR", "Warrior"},
    {"DEATHKNIGHT", "Death Knight"},
    {"MONK", "Monk"}
};

local taskTypes = {"Daily", "Weekly"};

local DA = DailyAzeroth;

--
-- Local Functions
--

local function ColourString(input, colour)
	return "|c" .. colour.colorStr .. input .. FONT_COLOR_CODE_CLOSE;
end

local function ResetUserData()
	DailyAzerothData = {
		Filters = {},
		Tracks = {}
	};
end

local function RegisterSortTab(tab, sortType)
	table.insert(DailyAzeroth.SortTabs, tab);
	tab.SortType = sortType;
end

local function RefreshButtons()
	if DA.SelectCount > 0 then
		DailyAzerothMainCompleteButton:Enable();
		DailyAzerothMainDeleteButton:Enable();
	else
		DailyAzerothMainCompleteButton:Disable();
		DailyAzerothMainDeleteButton:Disable();
	end
end

local function Load()
	-- Init user data if none exists.
	if DailyAzerothData == nil then
		ResetUserData();
	end
	
	for index, track in pairs(DailyAzerothData.Tracks) do
		track.Selected = false;
	end
	
	RegisterSortTab(DailyAzerothTitleSort, "Name");
	RegisterSortTab(DailyAzerothCharacterSort, "Character");
	RegisterSortTab(DailyAzerothNoteSort, "Note");
	RegisterSortTab(DailyAzerothTypeSort, "Type");
	
	RefreshButtons();
end

local function UpdateSelectCount()
	DailyAzerothMainSelectedText:SetText(DA.SelectCount .. " tasks selected.");
end

local function GetNonCompletedTracks()
	local tracks = DailyAzerothData.Tracks;
	local newTracks = {};
	
	for index, track in pairs(tracks) do
		if not track.Completed then
			table.insert(newTracks, track);
		end
	end
	
	return newTracks;
end

local function FilterExists(charName)
	for index, value in pairs(DailyAzerothData.Filters) do
		if value.Name == charName then
			return true;
		end
	end
	
	return false;
end

local function SlideOut_Clear()
	-- New filter name field.
	DailyAzerothSlideOutNewFilterNameField:Hide();
	DailyAzerothSlideOutNewFilterNameLabel:Hide();
	
	-- New filter class drop down.
	DailyAzerothSlideOutFilterClassDropDown:Hide();
	DailyAzerothSlideOutNewFilterClassLabel:Hide();
	
	-- New filter creation button.
	DailyAzerothSlideOutNewFilterCreateButton:Hide();
	
	-- New task name field.
	DailyAzerothSlideOutNewTaskTitleField:Hide();
	DailyAzerothSlideOutNewTaskTitleLabel:Hide();
	
	-- New task character field.
	DailyAzerothSlideOutNewTaskCharacterField:Hide();
	DailyAzerothSlideOutNewTaskCharacterLabel:Hide();
	
	-- New task note field.
	DailyAzerothSlideOutNewTaskNoteField:Hide();
	DailyAzerothSlideOutNewTaskNoteLabel:Hide();
	
	-- New task type field.
	DailyAzerothSlideOutNewTaskTypeField:Hide();
	DailyAzerothSlideOutNewTaskTypeLabel:Hide();
	
	-- New task icon field.
	DailyAzerothSlideOutNewTaskIconField:Hide();
	DailyAzerothSlideOutNewTaskIconLabel:Hide();
	
	-- New task creation button.
	DailyAzerothSlideOutNewTaskCreateButton:Hide();
end

local function SlideOut_NewFilter()
	SlideOut_Clear();
	
	-- Name field.
	DailyAzerothSlideOutNewFilterNameField:SetText("");
	DailyAzerothSlideOutNewFilterNameField:Show();
	DailyAzerothSlideOutNewFilterNameLabel:Show();
	
	-- Class drop down
	DailyAzerothSlideOutFilterClassDropDown:Show();
	DailyAzerothSlideOutNewFilterClassLabel:Show();
	
	-- Creation button
	DailyAzerothSlideOutNewFilterCreateButton:Show();
	
	DailyAzerothSlideOut:Show();
end

local function SlideOut_NewTask()
	SlideOut_Clear();
	
	-- Title field.
	DailyAzerothSlideOutNewTaskTitleField:SetText("");
	DailyAzerothSlideOutNewTaskTitleField:Show();
	DailyAzerothSlideOutNewTaskTitleLabel:Show();
	
	-- Character field.
	DailyAzerothSlideOutNewTaskCharacterField:Show();
	DailyAzerothSlideOutNewTaskCharacterLabel:Show();
	
	-- Note field
	DailyAzerothSlideOutNewTaskNoteField:Show();
	DailyAzerothSlideOutNewTaskNoteLabel:Show();
	
	-- Type field
	DailyAzerothSlideOutNewTaskTypeField:Show();
	DailyAzerothSlideOutNewTaskTypeLabel:Show();
	
	-- Icon field.
	DailyAzerothSlideOutNewTaskIconField:Show();
	DailyAzerothSlideOutNewTaskIconLabel:Show();
	
	-- Creation button
	DailyAzerothSlideOutNewTaskCreateButton:Show();
	
	DailyAzerothSlideOut:Show();
end

local function FilterTracks(tracks)
	local newTracks = {};
	
	for index, value in pairs(tracks) do
		if value ~= nil and (DA.SelectedFilter == "All" or value.Character == DA.SelectedFilter) then
			table.insert(newTracks, value);
		end
	end
	
	return newTracks;
end

--
-- Global Functions
--

function DailyAzeroth_DeleteSelectedFilter()
	if DA.SelectedFilter ~= "All" then
		-- Delete this filter.
		for index, filter in pairs(DailyAzerothData.Filters) do
			if filter.Name == DA.SelectedFilter then
				DailyAzerothData.Filters[index] = nil;
			end
		end
		
		-- Delete tracks using this filter
		for index, track in pairs(DailyAzerothData.Tracks) do
			if track.Character == DA.SelectedFilter then
				DailyAzerothData.Tracks[index] = nil;
			end
		end
		
		-- Select the "All" filter.
		DailyAzerothFilterButton1:Click();
	end
end

function DailyAzeroth_ResetTasks(taskType)
	for index, track in pairs(DailyAzerothData.Tracks) do
		if track.Type == taskType then
			track.Completed = false;
		end
	end
	
	DailyAzeroth_UpdateButtons();
end

function DailyAzeroth_CreateNewTask()
	local taskName = DailyAzerothSlideOutNewTaskTitleField:GetText();
	local filterIndex = DailyAzerothSlideOutNewTaskCharacterField.selectedValue;
	local note = DailyAzerothSlideOutNewTaskNoteField:GetText();
	local typeIndex = DailyAzerothSlideOutNewTaskTypeField.selectedValue;
	local icon = DailyAzerothSlideOutNewTaskIconField:GetText();
	
	if string.len(icon) > 0 and string.len(taskName) > 0 and typeIndex > 0 then
		local filters = {};
	
		if filterIndex == 0 then
			for index, filter in pairs(DailyAzerothData.Filters) do
				table.insert(filters, filter.Name);
			end
		else		
			local filter = DailyAzerothData.Filters[filterIndex];
			if filter == nil then
				return;
			end
			
			table.insert(filters, filter.Name);
		end
		
		for index, character in pairs(filters) do
				local newTask = {
				Name = taskName,
				Character = character,
				Note = note,
				Type = taskTypes[typeIndex],
				Icon = icon,
				Completed = false,
				Selected = false
			};
			
			table.insert(DailyAzerothData.Tracks, newTask);
		end
		
		DailyAzeroth_UpdateButtons();
		DailyAzerothSlideOut:Hide();
	end
end

function DailyAzeroth_CreateNewFilter()
	local classValue = DailyAzerothSlideOutFilterClassDropDown.selectedValue;
	local characterName = DailyAzerothSlideOutNewFilterNameField:GetText();
	
	if classValue > 0 and string.len(characterName) > 0 then
		if not FilterExists(characterName) then
			local newFilter = {
				Name = characterName,
				Colour = RAID_CLASS_COLORS[classes[classValue][1]];
			};
			
			table.insert(DailyAzerothData.Filters, newFilter);
			DailyAzeroth_UpdateFilters();
			DailyAzerothSlideOut:Hide();
		end
	end
end

function DailyAzeroth_InitTypeDropdown()
	for index, value in pairs(taskTypes) do
		local info = UIDropDownMenu_CreateInfo();
		
		info.text = value;
		info.value = index;
		info.func = DailyAzeroth_ClickFilterDropdown;
		
		UIDropDownMenu_AddButton(info);
	end
end

function DailyAzeroth_InitCharacterDropdown()
	local info = UIDropDownMenu_CreateInfo();
	
	info.text = "All Characters";
	info.value = 0;
	info.func = DailyAzeroth_ClickFilterDropdown;
	
	UIDropDownMenu_AddButton(info);
	
	for index, value in pairs(DailyAzerothData.Filters) do
		info = UIDropDownMenu_CreateInfo();
		
		info.text = ColourString(value.Name, value.Colour);
		info.value = index;
		info.func = DailyAzeroth_ClickFilterDropdown;
		
		UIDropDownMenu_AddButton(info);
	end
end

function DailyAzeroth_InitFilterDropdown()
	for index, value in pairs(classes) do
		local info = UIDropDownMenu_CreateInfo();
		
		info.text = ColourString(value[2], RAID_CLASS_COLORS[value[1]]);
		info.value = index;
		info.func = DailyAzeroth_ClickFilterDropdown;
		
		UIDropDownMenu_AddButton(info);
	end
end

function DailyAzeroth_ClickFilterDropdown(self, arg1, arg2, checked)
	if not checked then
		UIDropDownMenu_SetSelectedValue(UIDROPDOWNMENU_OPEN_MENU, self.value);
	end
end

function DailyAzeroth_NewFilterButtonClick()
	DailyAzerothSlideOutTitle:SetText("New Character");
	SlideOut_NewFilter();
end

function DailyAzeroth_NewTaskButtonClick()
	DailyAzerothSlideOutTitle:SetText("New Task");
	SlideOut_NewTask();
end

function DailyAzeroth_ToggleCompleted(button)
	DA.ShowCompleted = button:GetChecked();
	DailyAzeroth_UpdateButtons();
end

function DailyAzeroth_CompleteSelected()
	for index, track in pairs(DailyAzerothData.Tracks) do
		if track.Selected then
			track.Completed = true;
			
			if track.Selected then
				track.Selected = false;
				DA.SelectCount = DA.SelectCount - 1;
				RefreshButtons();
			end
		end
	end
	
	UpdateSelectCount();
	DailyAzeroth_UpdateButtons();
end

function DailyAzeroth_DeleteSelected()
	for index, track in pairs(DailyAzerothData.Tracks) do
		if track.Selected then
			DailyAzerothData.Tracks[index] = nil;
			DA.SelectCount = DA.SelectCount - 1;
			RefreshButtons();
		end
	end
	
	UpdateSelectCount();
	DailyAzeroth_UpdateButtons();
end

function DailyAzeroth_OnFilterClick(button, filter)
	DA.SelectedFilter = filter.Name;
	DailyAzeroth_UpdateFilters();
	DailyAzeroth_UpdateButtons();
end

function DailyAzeroth_OnClickSortColumn(button)
	if DA.Sorting == button.SortType then
		-- We're already sorting by this, switch the sort order.
		if DA.SortMethod == 1 then
			DA.SortMethod = 2;
		else
			DA.SortMethod = 1;
		end
	else
		DA.Sorting = button.SortType;
	end
	
	DailyAzeroth_UpdateButtons();
	
	-- Update the arrow graphics.
	for index, tab in pairs(DA.SortTabs) do
		DailyAzeroth_CheckSortColumn(tab);
	end
end

function DailyAzeroth_CheckSortColumn(button)
	local arrow = _G[button:GetName() .. "Arrow"];
	
	if DA.Sorting == button.SortType then
		arrow:Show();
		if DA.SortMethod == 1 then
			arrow:SetTexCoord(0, 0.5625, 1.0, 0);
		else
			arrow:SetTexCoord(0, 0.5625, 0, 1.0);
		end
	else
		arrow:Hide();
	end
end

function DailyAzeroth_ClearButtonHighlights()
	for i = 1, 8 do
		local button = _G["DailyAzerothButton" .. i];
		button:UnlockHighlight();
		button.Selected = false;
	end
end

function DailyAzeroth_UpdateHighlight(button, track)
	if track.Selected then
		button:LockHighlight();
	else
		button:UnlockHighlight();
	end
end

function DailyAzeroth_OnButtonClick(button, track)
	if track.Selected then
		DA.SelectCount = DA.SelectCount - 1;
		track.Selected = false;
	else
		DA.SelectCount = DA.SelectCount + 1;
		track.Selected = true;
	end
	
	RefreshButtons();
	DailyAzeroth_UpdateHighlight(button, track);
	UpdateSelectCount();
end

function DailyAzeroth_UpdateButtons()
	local tracks = {};
	
	if DA.ShowCompleted then
		tracks = DailyAzerothData.Tracks;
	else
		tracks = GetNonCompletedTracks();
	end
	
	tracks = FilterTracks(tracks);

	table.sort(tracks, function(a, b)
		if DA.SortMethod == 1 then
			return a[DA.Sorting] < b[DA.Sorting];
		else
			return a[DA.Sorting] > b[DA.Sorting];
		end
	end);
	
	local trackCount = #tracks;
	local index = FauxScrollFrame_GetOffset(DailyAzerothScrollFrame);
	
	-- Loop all of the buttons.
	for i = 1, 8 do
		local track = tracks[index + 1];
		local button = _G["DailyAzerothButton" .. i];
		
		if track ~= nil then
			local nameField = _G[button:GetName() .. "Name"];
			local characterField = _G[button:GetName() .. "Character"];
			local noteField = _G[button:GetName() .. "Note"];
			local typeField = _G[button:GetName() .. "Type"];
			
			local icon = _G[button:GetName() .. "ItemIconTexture"];
			
			DailyAzeroth_UpdateHighlight(button, track);
			
			icon:SetTexture("Interface/Icons/" .. track.Icon);
			
			if track.Completed then
				nameField:SetText(GREEN_FONT_COLOR_CODE .. track.Name .. FONT_COLOR_CODE_CLOSE);
				characterField:SetText(GREEN_FONT_COLOR_CODE .. track.Character .. FONT_COLOR_CODE_CLOSE);
				noteField:SetText(GREEN_FONT_COLOR_CODE .. track.Note .. FONT_COLOR_CODE_CLOSE);
				typeField:SetText(GREEN_FONT_COLOR_CODE .. track.Type .. FONT_COLOR_CODE_CLOSE);
			else				
				nameField:SetText(track.Name);			
				characterField:SetText(track.Character);
				noteField:SetText(track.Note);
				typeField:SetText(track.Type);
			end
			
			if track.Selected then
				button:LockHighlight();
			else
				button:UnlockHighlight();
			end
			
			button.Track = track;
			button:Show();
		else
			button:Hide();
		end
		
		index = index + 1;
	end
	
	FauxScrollFrame_Update(DailyAzerothScrollFrame, trackCount, 8, 37);
end

function DailyAzeroth_UpdateFilters()
	local filterCount = #DailyAzerothData.Filters;
	local index = FauxScrollFrame_GetOffset(DailyAzerothFilterScrollFrame);
	
	-- Loop all of the filter buttons.
	for i = 1, 15 do
		local filter = nil;
		local button = _G["DailyAzerothFilterButton" .. i];
		
		if index == 0 then
			filter = defaultFilter;
		else
			filter = DailyAzerothData.Filters[index];
		end
		
		if filter ~= nil then		
			-- Check if we have enough buttons to make the scroll frame
			-- appear and shrink the buttons to prevent overlapping.
			if filterCount > 15 then
				button:SetWidth(136);
			else
				button:SetWidth(156);
			end
			
			local normalText = _G[button:GetName() .. "NormalText"];
			local normalTexture = _G[button:GetName() .. "NormalTexture"];
			local line = _G[button:GetName() .. "Lines"];
			
			button:SetText(ColourString(filter.Name, filter.Colour));
			normalText:SetPoint("LEFT", button, "LEFT", 20, 0);
			normalTexture:SetAlpha(0.0);    
			
			if filterCount == index then
				line:SetTexCoord(0.4375, 0.875, 0, 0.625);
			else
				line:SetTexCoord(0, 0.4375, 0, 0.625);
			end
			
			if filter.Name == DA.SelectedFilter then
				button:LockHighlight();
				button.Selected = true;
			else
				button:UnlockHighlight();
				button.Selected = false;
			end
			
			line:Show();
			button:Show();
			button.Filter = filter;
		else
			button:Hide();
		end
		
		index = index + 1;
	end
	
	FauxScrollFrame_Update(DailyAzerothFilterScrollFrame, filterCount, 15, 20);
end

function FilterButton_SetType(button, type, text, isLast)
    local normalText = _G[button:GetName().."NormalText"];
    local normalTexture = _G[button:GetName().."NormalTexture"];
    local line = _G[button:GetName().."Lines"];
    if ( type == "class" ) then
        button:SetText(text);
        normalText:SetPoint("LEFT", button, "LEFT", 4, 0);
        normalTexture:SetAlpha(1.0);    
        line:Hide();
    elseif ( type == "subclass" ) then
        button:SetText(HIGHLIGHT_FONT_COLOR_CODE..text..FONT_COLOR_CODE_CLOSE);
        normalText:SetPoint("LEFT", button, "LEFT", 12, 0);
        normalTexture:SetAlpha(0.4);
        line:Hide();
    elseif ( type == "invtype" ) then
        button:SetText(HIGHLIGHT_FONT_COLOR_CODE..text..FONT_COLOR_CODE_CLOSE);
        normalText:SetPoint("LEFT", button, "LEFT", 20, 0);
        normalTexture:SetAlpha(0.0);    
        if ( isLast ) then
            line:SetTexCoord(0.4375, 0.875, 0, 0.625);
        else
            line:SetTexCoord(0, 0.4375, 0, 0.625);
        end
        line:Show();
    end
    button.type = type; 
end

--
-- Event Handling
--

local eventFrame = CreateFrame("FRAME");
eventFrame:RegisterEvent("ADDON_LOADED");

eventFrame:SetScript("OnEvent", function(sender, event, ...)
	if event == "ADDON_LOADED" then
		local addonName = ...;
		if addonName == DA.Name then
			Load();
		end
	end
end);

--
-- Command Handling
--

SLASH_DA1, SLASH_DA2 = '/da', '/dailyazeroth';

local function handleCommand(msg, editbox)
	ShowUIPanel(DailyAzerothFrame);
end

SlashCmdList["DA"] = handleCommand;